@@ -1,7 +1,7 @@
 /* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
 
-#ifndef BLAKE2S_H
-#define BLAKE2S_H
+#ifndef _WG_BLAKE2S_H
+#define _WG_BLAKE2S_H
 
 #include <linux/types.h>
 
@@ -35,4 +35,4 @@ void blake2s_fpu_init(void);
 bool blake2s_selftest(void);
 #endif
 
-#endif
+#endif /* _WG_BLAKE2S_H */
